@@ -447,11 +447,15 @@ cvar_t cl_nopred = {CVAR_SAVE, "cl_nopred", "0", "(QWSV only) disables player mo
 cvar_t in_pitch_min = {0, "in_pitch_min", "-90", "how far you can aim upward (quake used -70)"};
 cvar_t in_pitch_max = {0, "in_pitch_max", "90", "how far you can aim downward (quake used 80)"};
 
-cvar_t m_filter = {CVAR_SAVE, "m_filter","0", "smoothes mouse movement, less responsive but smoother aiming"};
-cvar_t m_accelerate = {CVAR_SAVE, "m_accelerate","1", "mouse acceleration factor (try 2)"};
-cvar_t m_accelerate_minspeed = {CVAR_SAVE, "m_accelerate_minspeed","5000", "below this speed, no acceleration is done"};
-cvar_t m_accelerate_maxspeed = {CVAR_SAVE, "m_accelerate_maxspeed","10000", "above this speed, full acceleration is done"};
-cvar_t m_accelerate_filter = {CVAR_SAVE, "m_accelerate_filter","0.1", "mouse acceleration factor filtering"};
+cvar_t m_filter = {CVAR_SAVE, "m_filter","0", "smoothes mouse movement, less responsive but smoother aiming"}; 
+cvar_t m_accelerate = {CVAR_SAVE, "m_accelerate","1", "linear mouse acceleration factor (set to 1 to disable the linear acceleration and use only the power acceleration; set to 0 to disable all acceleration)"};
+cvar_t m_accelerate_minspeed = {CVAR_SAVE, "m_accelerate_minspeed","5000", "below this speed in px/s, no acceleration is done, with a linear slope between (applied only on linear acceleration)"};
+cvar_t m_accelerate_maxspeed = {CVAR_SAVE, "m_accelerate_maxspeed","10000", "above this speed in px/s, full acceleration is done, with a linear slope between (applied only on linear acceleration)"};
+cvar_t m_accelerate_filter = {CVAR_SAVE, "m_accelerate_filter","0", "linear mouse acceleration factor filtering lowpass constant in seconds (set to 0 for no filtering)"};
+cvar_t m_accelerate_power_offset = {CVAR_SAVE, "m_accelerate_power_offset","0", "below this speed in px/ms, no power acceleration is done"};
+cvar_t m_accelerate_power = {CVAR_SAVE, "m_accelerate_power","2", "acceleration power (must be above 1 to be useful)"};
+cvar_t m_accelerate_power_senscap = {CVAR_SAVE, "m_accelerate_power_senscap", "0", "maximum acceleration factor generated by power acceleration; use 0 for unbounded"};
+cvar_t m_accelerate_power_strength = {CVAR_SAVE, "m_accelerate_power_strength", "0", "strength of the power mouse acceleration effect"};
 
 cvar_t cl_netfps = {CVAR_SAVE, "cl_netfps","72", "how many input packets to send to server each second"};
 cvar_t cl_netrepeatinput = {CVAR_SAVE, "cl_netrepeatinput", "1", "how many packets in a row can be lost without movement issues when using cl_movement (technically how many input messages to repeat in each packet that have not yet been acknowledged by the server), only affects DP7 and later servers (Quake uses 0, QuakeWorld uses 2, and just for comparison Quake3 uses 1)"};
@@ -589,38 +593,88 @@ void CL_Input (void)
 		}
 	}
 
-	// apply m_accelerate if it is on
-	if(m_accelerate.value > 1)
+       // apply m_accelerate if it is on
+	if(m_accelerate.value > 0)
 	{
+		float mouse_deltadist = sqrtf(in_mouse_x * in_mouse_x + in_mouse_y * in_mouse_y);
+		float speed = mouse_deltadist / cl.realframetime;
 		static float averagespeed = 0;
-		float speed, f, mi, ma;
-
-		speed = sqrt(in_mouse_x * in_mouse_x + in_mouse_y * in_mouse_y) / cl.realframetime;
+		float f, mi, ma;
 		if(m_accelerate_filter.value > 0)
-			f = bound(0, cl.realframetime / m_accelerate_filter.value, 1);
+		        f = bound(0, cl.realframetime / m_accelerate_filter.value, 1);
 		else
-			f = 1;
+		        f = 1;
 		averagespeed = speed * f + averagespeed * (1 - f);
 
-		mi = max(1, m_accelerate_minspeed.value);
-		ma = max(m_accelerate_minspeed.value + 1, m_accelerate_maxspeed.value);
+		// Note: this check is technically unnecessary, as everything in here cancels out if it is zero.
+		if (m_accelerate.value != 1.0f)
+		{
+			// First do linear slope acceleration which was ripped "in
+			// spirit" from many classic mouse driver implementations.
+			// If m_accelerate.value == 1, this code does nothing at all.
 
-		if(averagespeed <= mi)
-		{
-			f = 1;
-		}
-		else if(averagespeed >= ma)
-		{
-			f = m_accelerate.value;
-		}
-		else
-		{
-			f = averagespeed;
-			f = (f - mi) / (ma - mi) * (m_accelerate.value - 1) + 1;
+			mi = max(1, m_accelerate_minspeed.value);
+			ma = max(m_accelerate_minspeed.value + 1, m_accelerate_maxspeed.value);
+
+			if(averagespeed <= mi)
+			{
+				f = 1;
+			}
+			else if(averagespeed >= ma)
+			{
+				f = m_accelerate.value;
+			}
+			else
+			{
+				f = averagespeed;
+				f = (f - mi) / (ma - mi) * (m_accelerate.value - 1) + 1;
+			}
+			in_mouse_x *= f;
+			in_mouse_y *= f;
 		}
 
-		in_mouse_x *= f;
-		in_mouse_y *= f;
+		// Note: this check is technically unnecessary, as everything in here cancels out if it is zero.
+		if (m_accelerate_power_strength.value != 0.0f)
+		{
+			// Then do Quake Live-style power acceleration.
+			// Note that this behavior REPLACES the usual
+			// sensitivity, so we apply it but then dividie by
+			// sensitivity.value so that the later multiplication
+			// restores it again.
+			float accelsens = 1.0f;
+			float adjusted_speed_pxms = (averagespeed * 0.001f - m_accelerate_power_offset.value) * m_accelerate_power_strength.value;
+			float inv_sensitivity = 1.0f / sensitivity.value;
+			if (adjusted_speed_pxms > 0)
+			{
+				if (m_accelerate_power.value > 1.0f)
+				{
+					// TODO: How does this interact with sensitivity changes? Is this intended?
+					// Currently: more sensitivity = less acceleration at same pixel speed.
+					accelsens += expf((m_accelerate_power.value - 1.0f) * logf(adjusted_speed_pxms)) * inv_sensitivity;
+				}
+				else
+				{
+					// The limit of the then-branch for m_accelerate_power -> 1.
+					accelsens += inv_sensitivity;
+					// Note: QL had just accelsens = 1.0f.
+					// This is mathematically wrong though.
+				}
+			}
+			else
+			{
+				// The limit of the then-branch for adjusted_speed -> 0.
+				// accelsens += 0.0f;
+			}
+			if (m_accelerate_power_senscap.value > 0.0f && accelsens > m_accelerate_power_senscap.value * inv_sensitivity)
+			{
+				// TODO: How does this interact with sensitivity changes? Is this intended?
+				// Currently: senscap is in absolute sensitivity units, so if senscap < sensitivity, it overrides.
+			        accelsens = m_accelerate_power_senscap.value * inv_sensitivity;
+			}
+
+			in_mouse_x *= accelsens;
+			in_mouse_y *= accelsens;
+		}
 	}
 
 	// apply m_filter if it is on
@@ -839,7 +893,7 @@ static qboolean CL_ClientMovement_Unstick(cl_clientmovement_state_t *s)
 	for (i = 0;i < NUMOFFSETS;i++)
 	{
 		VectorAdd(offsets[i], s->origin, neworigin);
-		if (!CL_TraceBox(neworigin, cl.playercrouchmins, cl.playercrouchmaxs, neworigin, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true).startsolid)
+		if (!CL_TraceBox(neworigin, cl.playercrouchmins, cl.playercrouchmaxs, neworigin, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true).startsolid)
 		{
 			VectorCopy(neworigin, s->origin);
 			return true;
@@ -871,7 +925,7 @@ static void CL_ClientMovement_UpdateStatus(cl_clientmovement_state_t *s)
 		// low ceiling first
 		if (s->crouched)
 		{
-			trace = CL_TraceBox(s->origin, cl.playerstandmins, cl.playerstandmaxs, s->origin, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true);
+			trace = CL_TraceBox(s->origin, cl.playerstandmins, cl.playerstandmaxs, s->origin, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true);
 			if (!trace.startsolid)
 				s->crouched = false;
 		}
@@ -890,7 +944,7 @@ static void CL_ClientMovement_UpdateStatus(cl_clientmovement_state_t *s)
 	// set onground
 	VectorSet(origin1, s->origin[0], s->origin[1], s->origin[2] + 1);
 	VectorSet(origin2, s->origin[0], s->origin[1], s->origin[2] - 1); // -2 causes clientside doublejump bug at above 150fps, raising that to 300fps :)
-	trace = CL_TraceBox(origin1, s->mins, s->maxs, origin2, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true);
+	trace = CL_TraceBox(origin1, s->mins, s->maxs, origin2, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true);
 	if(trace.fraction < 1 && trace.plane.normal[2] > 0.7)
 	{
 		s->onground = true;
@@ -906,16 +960,16 @@ static void CL_ClientMovement_UpdateStatus(cl_clientmovement_state_t *s)
 	// set watertype/waterlevel
 	VectorSet(origin1, s->origin[0], s->origin[1], s->origin[2] + s->mins[2] + 1);
 	s->waterlevel = WATERLEVEL_NONE;
-	s->watertype = CL_TracePoint(origin1, MOVE_NOMONSTERS, s->self, 0, 0, true, false, NULL, false).startsupercontents & SUPERCONTENTS_LIQUIDSMASK;
+	s->watertype = CL_TracePoint(origin1, MOVE_NOMONSTERS, s->self, 0, 0, 0, true, false, NULL, false).startsupercontents & SUPERCONTENTS_LIQUIDSMASK;
 	if (s->watertype)
 	{
 		s->waterlevel = WATERLEVEL_WETFEET;
 		origin1[2] = s->origin[2] + (s->mins[2] + s->maxs[2]) * 0.5f;
-		if (CL_TracePoint(origin1, MOVE_NOMONSTERS, s->self, 0, 0, true, false, NULL, false).startsupercontents & SUPERCONTENTS_LIQUIDSMASK)
+		if (CL_TracePoint(origin1, MOVE_NOMONSTERS, s->self, 0, 0, 0, true, false, NULL, false).startsupercontents & SUPERCONTENTS_LIQUIDSMASK)
 		{
 			s->waterlevel = WATERLEVEL_SWIMMING;
 			origin1[2] = s->origin[2] + 22;
-			if (CL_TracePoint(origin1, MOVE_NOMONSTERS, s->self, 0, 0, true, false, NULL, false).startsupercontents & SUPERCONTENTS_LIQUIDSMASK)
+			if (CL_TracePoint(origin1, MOVE_NOMONSTERS, s->self, 0, 0, 0, true, false, NULL, false).startsupercontents & SUPERCONTENTS_LIQUIDSMASK)
 				s->waterlevel = WATERLEVEL_SUBMERGED;
 		}
 	}
@@ -942,20 +996,20 @@ static void CL_ClientMovement_Move(cl_clientmovement_state_t *s)
 	for (bump = 0, t = s->cmd.frametime;bump < 8 && VectorLength2(s->velocity) > 0;bump++)
 	{
 		VectorMA(s->origin, t, s->velocity, neworigin);
-		trace = CL_TraceBox(s->origin, s->mins, s->maxs, neworigin, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true);
+		trace = CL_TraceBox(s->origin, s->mins, s->maxs, neworigin, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true);
 		if (trace.fraction < 1 && trace.plane.normal[2] == 0)
 		{
 			// may be a step or wall, try stepping up
 			// first move forward at a higher level
 			VectorSet(currentorigin2, s->origin[0], s->origin[1], s->origin[2] + cl.movevars_stepheight);
 			VectorSet(neworigin2, neworigin[0], neworigin[1], s->origin[2] + cl.movevars_stepheight);
-			trace2 = CL_TraceBox(currentorigin2, s->mins, s->maxs, neworigin2, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true);
+			trace2 = CL_TraceBox(currentorigin2, s->mins, s->maxs, neworigin2, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true);
 			if (!trace2.startsolid)
 			{
 				// then move down from there
 				VectorCopy(trace2.endpos, currentorigin2);
 				VectorSet(neworigin2, trace2.endpos[0], trace2.endpos[1], s->origin[2]);
-				trace3 = CL_TraceBox(currentorigin2, s->mins, s->maxs, neworigin2, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true);
+				trace3 = CL_TraceBox(currentorigin2, s->mins, s->maxs, neworigin2, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true);
 				//Con_Printf("%f %f %f %f : %f %f %f %f : %f %f %f %f\n", trace.fraction, trace.endpos[0], trace.endpos[1], trace.endpos[2], trace2.fraction, trace2.endpos[0], trace2.endpos[1], trace2.endpos[2], trace3.fraction, trace3.endpos[0], trace3.endpos[1], trace3.endpos[2]);
 				// accept the new trace if it made some progress
 				if (fabs(trace3.endpos[0] - trace.endpos[0]) >= 0.03125 || fabs(trace3.endpos[1] - trace.endpos[1]) >= 0.03125)
@@ -1009,10 +1063,10 @@ static void CL_ClientMovement_Physics_Swim(cl_clientmovement_state_t *s)
 		AngleVectors(yawangles, forward, NULL, NULL);
 		VectorMA(s->origin, 24, forward, spot);
 		spot[2] += 8;
-		if (CL_TracePoint(spot, MOVE_NOMONSTERS, s->self, 0, 0, true, false, NULL, false).startsolid)
+		if (CL_TracePoint(spot, MOVE_NOMONSTERS, s->self, 0, 0, 0, true, false, NULL, false).startsolid)
 		{
 			spot[2] += 24;
-			if (!CL_TracePoint(spot, MOVE_NOMONSTERS, s->self, 0, 0, true, false, NULL, false).startsolid)
+			if (!CL_TracePoint(spot, MOVE_NOMONSTERS, s->self, 0, 0, 0, true, false, NULL, false).startsolid)
 			{
 				VectorScale(forward, 50, s->velocity);
 				s->velocity[2] = 310;
@@ -1355,9 +1409,9 @@ static void CL_ClientMovement_Physics_Walk(cl_clientmovement_state_t *s)
 				VectorSet(neworigin2, s->origin[0] + s->velocity[0]*(16/f), s->origin[1] + s->velocity[1]*(16/f), s->origin[2] + s->mins[2]);
 				VectorSet(neworigin3, neworigin2[0], neworigin2[1], neworigin2[2] - 34);
 				if (cls.protocol == PROTOCOL_QUAKEWORLD)
-					trace = CL_TraceBox(neworigin2, s->mins, s->maxs, neworigin3, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true);
+					trace = CL_TraceBox(neworigin2, s->mins, s->maxs, neworigin3, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true);
 				else
-					trace = CL_TraceLine(neworigin2, neworigin3, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, collision_extendmovelength.value, true, true, NULL, true, false);
+					trace = CL_TraceLine(neworigin2, neworigin3, MOVE_NORMAL, s->self, SUPERCONTENTS_SOLID | SUPERCONTENTS_BODY | SUPERCONTENTS_PLAYERCLIP, 0, 0, collision_extendmovelength.value, true, true, NULL, true, false);
 				if (trace.fraction == 1 && !trace.startsolid)
 					friction *= cl.movevars_edgefriction;
 			}
@@ -2263,6 +2317,10 @@ void CL_InitInput (void)
 	Cvar_RegisterVariable(&m_accelerate_minspeed);
 	Cvar_RegisterVariable(&m_accelerate_maxspeed);
 	Cvar_RegisterVariable(&m_accelerate_filter);
+	Cvar_RegisterVariable(&m_accelerate_power);
+	Cvar_RegisterVariable(&m_accelerate_power_offset);
+	Cvar_RegisterVariable(&m_accelerate_power_senscap);
+	Cvar_RegisterVariable(&m_accelerate_power_strength);
 
 	Cvar_RegisterVariable(&cl_netfps);
 	Cvar_RegisterVariable(&cl_netrepeatinput);
